/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.secure;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.*;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;

import java.util.Map;

/**
 * Handles user related operations in the registry. Checks if the actor is authorized to perform
 * the requested operation and delegated it to the realm, if authorization succeeds.
 */
public class RegistryUserManager {

    private static final Log log = LogFactory.getLog(RegistryUserManager.class);

    private Realm realm;
    private String actor;

    public RegistryUserManager(Realm realm, String userName ) {

        this.realm = realm;
        this.actor = userName;
    }

    public Realm getRealm() {
        return realm;
    }

    public void addUser(String userName,
                        String password) throws RegistryException {

        try {
            if (!realm.isUserAuthorized(
                    actor, RegistryConstants.ROOT_PATH, ActionConstants.ADD_USER)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.addUser(userName, password);
            //In this time we need to create a directory for users and added that to the registry.
        } catch (UserManagerException e) {
            String msg = "Could not add the user. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void addRole(String roleName) throws RegistryException {

        try {
            if (!realm.isUserAuthorized(
                    actor, RegistryConstants.ROOT_PATH, ActionConstants.ADD_ROLE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.addRole(roleName);
        } catch (UserManagerException e) {
            String msg = "Could not add the role. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void removeUser(String userName) throws RegistryException {

        if (userName.equals(RegistryConstants.ADMIN_USER)) {
            String msg = "Cannot remove the administrator.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            if (!realm.isUserAuthorized(
                    actor, RegistryConstants.ROOT_PATH, ActionConstants.REMOVE_USER)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.deleteUser(userName);
        } catch (UserManagerException e) {
            String msg = "Could not delete the user. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void removeRole(String roleName) throws RegistryException {

        if (roleName.equals(RegistryConstants.ADMIN_USER)) {
            String msg = "Cannot remove the administrator role.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            if (!realm.isUserAuthorized(
                    actor, RegistryConstants.ROOT_PATH, ActionConstants.REMOVE_ROLE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.deleteRole(roleName);
        } catch (UserManagerException e) {
            String msg = "Could not delete the role. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public String[] getAllUsers() throws RegistryException {

        try {
            return realm.getAllUserNames();
        } catch (UserManagerException e) {
            String msg = "Could not get all users of the Registry. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public String[] getAllRoles() throws RegistryException {

        try {
            return realm.getAllRoleNames();
        } catch (UserManagerException e) {
            String msg = "Could not get all roles of the Registry. Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public boolean userExists(String userName) throws RegistryException {

        String[] allUsers = getAllUsers();

        for (int i = 0; i < allUsers.length; i++) {
            if (userName.equals(allUsers[i])) {
                return true;
            }
        }

        return false;
    }

    public Map getUserProperties(String userName) throws RegistryException {
        Map properties ;
        try {
            properties = realm.getUserProperties(userName);
        } catch (UserManagerException e) {
            throw new RegistryException(e.getMessage());
        }
        return properties;
    }

    public void setUserProperties(String userName, Map properties) throws RegistryException {

        try {
            realm.setUserProperties(userName, properties);
        } catch (UserManagerException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public String getUserProperty(String userName, String propName) throws RegistryException {

        try {
            Map properties = realm.getUserProperties(userName);
            return (String) properties.get(propName);
        } catch (UserManagerException e) {
            throw new RegistryException(e.getMessage());
        }        
    }

    public void setUserProperty(String userName, String name, String value) throws RegistryException {

        try {
            Map properties = realm.getUserProperties(userName);
            properties.put(name, value);
            realm.setUserProperties(userName, properties);

        } catch (UserManagerException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public void authorizeUser(String userName, String resourceID, String action)
            throws RegistryException {

        try {
            if (!realm.isUserAuthorized(actor, resourceID, ActionConstants.AUTHORIZE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.authorizeUser(userName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not authorize. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void authorizeRole(String roleName, String resourceID, String action)
            throws RegistryException {

        try {
            if (!realm.isUserAuthorized(actor, resourceID, ActionConstants.AUTHORIZE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.authorizeRole(roleName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not authorize. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void denyUser(String userName, String resourceID, String action)
            throws RegistryException {

        if (userName.equals(RegistryConstants.ADMIN_USER)) {
            String msg = "Cannot deny access for the administrator.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            if (!realm.isUserAuthorized(actor, resourceID, ActionConstants.AUTHORIZE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.denyUser(userName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not deny. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void denyRole(String roleName, String resourceID, String action)
            throws RegistryException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {
            String msg = "Cannot deny access for the administrator role.";
            log.info(msg);
            throw new RegistryException(msg);
        }

        try {
            if (!realm.isUserAuthorized(actor, resourceID, ActionConstants.AUTHORIZE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.denyRole(roleName, resourceID, action);
        } catch (UserManagerException e) {
            String msg = "Could not deny. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public boolean isUserAuthorized(String userName, String resourcePath, String action)
            throws RegistryException {

        try {
            return realm.isUserAuthorized(userName, resourcePath, action);
        } catch (UserManagerException e) {
            String msg = "Could not get the permissions for the user " +
                    userName + " for the resource " + resourcePath;
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void addUserToRole(String userName, String roleName) throws RegistryException {

        try {
            if (!realm.isUserAuthorized(
                    actor, RegistryConstants.ROOT_PATH, ActionConstants.AUTHORIZE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.addUserToRole(userName, roleName);
        } catch (UserManagerException e) {
            String msg = "Could not add user " + userName + " to role " + roleName + ".";
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public void removeUserFromRole(String userName, String roleName) throws RegistryException {

        try {
            if (!realm.isUserAuthorized(
                    actor, RegistryConstants.ROOT_PATH, ActionConstants.AUTHORIZE)) {
                String msg = "Attempted to perform unauthorized operation.";
                log.info(msg);
                throw new RegistryException(msg);
            }
        } catch (UserManagerException e) {
            String msg = "Could not check authorization. \nCaused by " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }

        try {
            realm.removeUserFromRole(userName, roleName);
        } catch (UserManagerException e) {
            String msg = "Could not remove user " + userName + " from role " + roleName + ".";
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }
}
